// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#ifndef _XRM_DEC_INTERFACE_H_
#define _XRM_DEC_INTERFACE_H_
#ifdef __cplusplus
extern "C" {
#endif

#include <xrm.h>
#include "xrm_interface.h"

typedef struct XrmDecodeContext {
  int                 dev_index;
  int                 dec_load;
  int                 xrm_reserve_id;
  int                 decode_res_in_use;
  xrmContext          xrm_ctx;
  xrmCuListResourceV2 decode_cu_list_res;
} XrmDecodeContext;

/**
 * @brief Release the xrm resources which were allocated. Destroy the xrm
 * context.
 * 
 * @param xrm_dec_ctx: The xrm decoder context
 */
void xrm_dec_release(XrmDecodeContext* xrm_dec_ctx);

/**
 * @brief Check & allocate the necessary xrm resources given the relevant 
 * decoder parameters
 * 
 * @param xrm_dec_ctx The xrm wrapper struct for xrm information
 * @param dev_index The device which will be queried
 * @param xrm_props Contains relevant information for load calculation
 * @return XRM_SUCCESS on success, XRM_ERROR on error or XRM_ERROR_CONNECT_FAIL
 * when unable to connect to daemon
 */
int xrm_dec_reserve(XrmDecodeContext* xrm_dec_ctx, int dev_index, XrmInterfaceProperties* xrm_props);
#ifdef __cplusplus
}
#endif
#endif // _XRM_DEC_INTERFACE_H_
